@@ -6,23 +6,39 @@ use Exporter;
 @ISA = 'Exporter';
 @EXPORT_OK = qw/capture capture_exec qxx capture_exec_combined qxy/;
 %EXPORT_TAGS = (all => \@EXPORT_OK);
-$VERSION = '1.07_01';
+$VERSION = '1.08_50';
+$VERSION = eval $VERSION;
 
 sub capture (&@) { ## no critic
     my ($code, $output, $error, $output_file, $error_file) = @_;
-    for ($output, $error) {
-        $_ = \do { my $s; $s = ''} unless ref $_;
-        $$_ = '' if $_ != \undef && !defined($$_);
+
+    # if neither are defined, then capture to files only
+    if ( defined $output || defined $error ) {
+      for ($output, $error) {
+          $_ = \do { my $s; $s = ''} unless ref $_;
+          $$_ = '' if $_ != \undef && !defined($$_);
+      }
     }
+
+    # don't merge if both undef -- someone might still want to capture
+    # them separately in temp files
+    my $should_merge = 
+      (defined $error && defined $output && $output == $error) || 
+      ( !defined $output && !defined $error ) || 
+      0;
+
     my ($capture_out, $capture_err);
-    if ( $output != \undef ) { 
+
+    # undef means capture anonymously; anything other than \undef means 
+    # capture to that ref; \undef means skip capture
+    if ( !defined $output || $output != \undef ) { 
         $capture_out = IO::CaptureOutput::_proxy->new(
             'STDOUT', $output, undef, $output_file
         );
     }
-    if ( $error != \undef ) { 
+    if ( !defined $error || $error != \undef ) { 
         my $capture_err = IO::CaptureOutput::_proxy->new(
-            'STDERR', $error, ($output == $error ? 'STDOUT' : undef), $error_file
+            'STDERR', $error, ($should_merge ? 'STDOUT' : undef), $error_file
         );
     }
     &$code();
@@ -73,75 +89,76 @@ sub _is_wperl { $^O eq 'MSWin32' && basename($^X) eq 'wperl.exe' }
 
 sub new {
     my $class = shift;
-    my ($fh, $capture, $merge_fh, $capture_file) = @_;
-    $fh       = qualify($fh);         # e.g. main::STDOUT
-    my $fhref = qualify_to_ref($fh);  # e.g. \*STDOUT
+    my ($orig_fh, $capture_var, $merge_fh, $capture_file) = @_;
+    $orig_fh       = qualify($orig_fh);         # e.g. main::STDOUT
+    my $fhref = qualify_to_ref($orig_fh);  # e.g. \*STDOUT
 
     # Duplicate the filehandle
-    my $saved;
+    my $saved_fh;
     {
         no strict 'refs'; ## no critic - needed for 5.005
-        if ( defined fileno($fh) && ! _is_wperl() ) {
-            $saved = gensym;
-            open $saved, ">&$fh" or croak "Can't redirect <$fh> - $!";
+        if ( defined fileno($orig_fh) && ! _is_wperl() ) {
+            $saved_fh = gensym;
+            open $saved_fh, ">&$orig_fh" or croak "Can't redirect <$orig_fh> - $!";
         }
     }
 
     # Create replacement filehandle if not merging
-    my ($newio, $newio_file);
+    my ($newio_fh, $newio_file);
     if ( ! $merge_fh ) {
-        $newio = gensym;
+        $newio_fh = gensym;
         if ($capture_file) {
             $newio_file = $capture_file;
         } else {
             (undef, $newio_file) = tempfile;
         }
-        open $newio, "+>$newio_file" or croak "Can't create temp file for $fh - $!";
+        open $newio_fh, "+>$newio_file" or croak "Can't write temp file for $orig_fh - $!";
     }
     else {
-        $newio = qualify($merge_fh);
+        $newio_fh = qualify($merge_fh);
     }
 
     # Redirect (or merge)
     {
         no strict 'refs'; ## no critic -- needed for 5.005
-        open $fhref, ">&".fileno($newio) or croak "Can't redirect $fh - $!";
+        open $fhref, ">&".fileno($newio_fh) or croak "Can't redirect $orig_fh - $!";
     }
 
-    bless [$$, $fh, $saved, $capture, $newio, $newio_file, $capture_file], $class;
+    bless [$$, $orig_fh, $saved_fh, $capture_var, $newio_fh, $newio_file, $capture_file], $class;
 }
 
 sub DESTROY {
     my $self = shift;
 
-    my ($pid, $fh, $saved) = @{$self}[0..2];
+    my ($pid, $orig_fh, $saved_fh, $capture_var, $newio_fh, 
+      $newio_file, $capture_file) = @$self;
     return unless $pid eq $$; # only cleanup in the process that is capturing
 
     # restore the original filehandle
-    my $fh_ref = Symbol::qualify_to_ref($fh);
+    my $fh_ref = Symbol::qualify_to_ref($orig_fh);
     select((select ($fh_ref), $|=1)[0]);
-    if (defined $saved) {
-        open $fh_ref, ">&". fileno($saved) or croak "Can't restore $fh - $!";
+    if (defined $saved_fh) {
+        open $fh_ref, ">&". fileno($saved_fh) or croak "Can't restore $orig_fh - $!";
     }
     else {
         close $fh_ref;
     }
 
     # transfer captured data to the scalar reference if we didn't merge
-    my ($capture, $newio, $newio_file) = @{$self}[3..5];
-    if ($newio_file) {
+    # $newio_file is undef if this file handle is merged to another
+    if (ref $capture_var && $newio_file) {
         # some versions of perl complain about reading from fd 1 or 2
         # which could happen if STDOUT and STDERR were closed when $newio
         # was opened, so we just squelch warnings here and continue
         local $^W; 
-        seek $newio, 0, 0;
-        $$capture = do {local $/; <$newio>};
-        close $newio;
+        seek $newio_fh, 0, 0;
+        $$capture_var = do {local $/; <$newio_fh>};
     }
+    close $newio_fh if $newio_file;
 
     # Cleanup
     return unless defined $newio_file && -e $newio_file;
-    return if $self->[6]; # the "temp" file was explicitly named
+    return if $capture_file; # the "temp" file was explicitly named
     unlink $newio_file or carp "Couldn't remove temp file '$newio_file' - $!";
 }
 
@@ -172,7 +189,7 @@ This documentation describes version %%VERSION%%.
         print "arguments: @_";
     }
 
-    capture sub {noisy(@args)}, \$stdout, \$stderr;
+    capture { noisy(@args) } \$stdout, \$stderr;
 
     ($stdout, $stderr) = capture_exec( 'perl', '-e', 
         'print "Hello"; print STDERR "World!"');
@@ -195,12 +212,18 @@ Captures everything printed to {STDOUT} and {STDERR} for the duration of
 {&subroutine}. {$stdout} and {$stderr} are optional scalars that will contain
 {STDOUT} and {STDERR} respectively. 
 
+{capture()} uses a code prototype so the first argument can be specified directly within 
+brackets if desired.
+
+    # shorthand with prototype
+    capture { print __PACKAGE__ } \$stdout, \$stderr;
+
 Returns the return value(s) of {&subroutine}. The sub is called in the same
 context as {capture()} was called e.g.:
 
-    @rv = capture sub {wantarray}; # returns true
-    $rv = capture sub {wantarray}; # returns defined, but not true
-    capture sub {wantarray};       # void, returns undef
+    @rv = capture { wantarray } ; # returns true
+    $rv = capture { wantarray } ; # returns defined, but not true
+    capture { wantarray };       # void, returns undef
 
 {capture()} is able to capture output from subprocesses and C code, which
 traditional {tie()} methods of output capture are unable to do.
@@ -220,7 +243,7 @@ If desired, specific file names may be provided instead as additional options.
     capture \&subroutine, \$stdout, \$stderr, $out_file, $err_file;
 
 Files provided will be clobbered, overwriting any previous data, but
-will persist after the call to {capture} for inspection or other manipulation.
+will persist after the call to {capture()} for inspection or other manipulation.
 
 By default, when no references are provided to hold STDOUT or STDERR, output
 is captured and silently discarded.
